#!/usr/bin/env python
import os
import sys
import unicodedata


tag_delim = set([' ', '/', '>'])
tag_close = set(['>'])
tag_open = set(['<'])
ltr_set = set(['Ll', 'Lm', 'Lu'])


def is_lomaji(s):
    u = unicode(s, encoding='utf-8')
    c = unicodedata.category(u[0])
    if c == 'Zs' and len(u) > 1:
        c = unicodedata.category(u[1])

    return c in ltr_set


def ahead(buf, index, c):
    if buf[index] != c:
        raise AssertionError(
            'Unexpected char: %s, expected: %s, excerpt: %s' % (
                (buf[index], c, buf[index-10:index+10])))
    return index + 1


def until(buf, index, charset):
    in_quote = False
    while True:
        if buf[index] in charset and not in_quote:
            return index
        if buf[index] == '"':
            in_quote = not in_quote

        index += 1


def parse(buf, index):
    start = ahead(buf, index, '<')
    delim = until(buf, index, tag_delim)
    tag_name = buf[start:delim]
    attrs = ""
    text = ""
    children = []

    while buf[delim] == ' ':
        next_delim = until(buf, delim + 1, tag_delim)
        attrs += buf[delim:next_delim]
        delim = next_delim

    if buf[delim] == '/':
        delim += 1
        index = ahead(buf, delim, '>')
        return index, [tag_name, attrs, text, children]

    # look ahead
    delim += 1
    if buf[delim] != '<':
        text_end = until(buf, delim, tag_open)
        text = buf[delim:text_end]
        delim = text_end

    while buf[delim + 1] != '/':
        new_index, node = parse(buf, delim)
        children.append(node)
        delim = new_index

    end = until(buf, delim + 2, tag_close)
    if tag_name != buf[delim + 2:end]:
        raise AssertionError('Mismatched ending')

    return end + 1, [tag_name, attrs, text, children]


def write_node(f, node):
    if len(node[2]) == 0 and len(node[3]) == 0:
        f.write("<%s%s/>" % (node[0], node[1]))
    else:
        f.write("<%s%s>" % (node[0], node[1]))
        if len(node[2]):
            f.write(node[2])
        for child in node[3]:
            write_node(f, child)
        f.write("</%s>" % node[0])


sep = '<w:r w:rsidRPr="00155C32"><w:rPr><w:rFonts w:ascii="Taigi Unicode" w:hAnsi="Taigi Unicode"/><w:b/><w:color w:themeColor="text1" w:themeTint="F2" w:val="0D0D0D"/><w:sz w:val="20"/><w:szCs w:val="20"/></w:rPr><w:t xml:space="preserve"> </w:t></w:r>'   # nopep8
_, sep_node = parse(sep, 0)


def find_wr(node, indent=0):
    occr = 0
    children = node[3]

    has_wr = any(child[0] == 'w:r' for child in children)

    # print "%s%s -> %s" % (" " * indent, node[0], has_wr)

    def is_next_wr_true(tl, i):
        s = len(tl)
        i += 1
        while i < s:
            if tl[i][0] == 'w:r':
                return tl[i][1]
            i += 1
        return False

    if not has_wr:
        for child in children:
            occr += find_wr(child, indent+1)
    else:
        # print "---"
        # for child in children:
        #     find_wr(child, indent+1)

        fld_char = False
        m = [('', False)] * len(children)

        # Scan phase
        for index, child in enumerate(children):
            m[index] = (child[0], False)

            if child[0] != 'w:r':
                pass
            elif any(x[0] == 'w:ruby' for x in child[3]):
                m[index] = (child[0], True)
            elif any(x[0] == 'w:fldChar' for x in child[3]):
                fld_char = not fld_char
                if fld_char:
                    m[index] = (child[0], True)
            else:
                for x in child[3]:
                    if x[0] == 'w:instrText':
                        pass
                    elif x[0] == 'w:t':
                        if is_lomaji(x[2]):
                            m[index] = (child[0], True)

        # Rewrite phase
        fld_char = False
        new_children = []
        for index, child in enumerate(children):
            if child[0] != 'w:r':
                new_children.append(child)
            elif any(x[0] == 'w:ruby' for x in child[3]):
                new_children.append(child)
                if is_next_wr_true(m, index):
                    new_children.append(sep_node)
                    occr += 1
            elif any(x[0] == 'w:fldChar' for x in child[3]):
                fld_char = not fld_char
                new_children.append(child)

                # If closed
                if not fld_char:
                    if is_next_wr_true(m, index):
                        new_children.append(sep_node)
                        occr += 1
            else:
                new_children.append(child)
                for x in child[3]:
                    if x[0] == 'w:t' and is_lomaji(x[2]):
                        if is_next_wr_true(m, index):
                            new_children.append(sep_node)
                            occr += 1

        node[3] = new_children

    return occr


def add_space(input, output):
    f = open(input)
    l1 = f.readline()
    l2 = f.readline()
    c = f.readline()
    f.close()

    _, root = parse(c, 0)

    occr = find_wr(root)

    f = open(output, 'w')
    f.writelines([l1, l2])
    write_node(f, root)
    f.close()
    return occr


if __name__ == '__main__':
    if len(sys.argv) != 2:
        print 'usage: addspace xmlfile'
        sys.exit(1)

    src = sys.argv[1]
    head, tail = os.path.split(src)
    base, ext = os.path.splitext(tail)
    dst = os.path.join(head, '%s-spaced-added%s' % (base, ext))

    occurences = add_space(src, dst)
    print "%i occurences updated, output file: %s" % (occurences, dst)
