#!/usr/bin/env python
import os
import re
import sys


def color_non_kanji_text(input, output, color):
    """Change color.

    Two examples. Normal text:

        <w:r w:rsidRPr="00FB4E90">
            <w:rPr>
                <w:rFonts w:ascii="Taigi Unicode" w:hAnsi="Taigi Unicode"/>
                <w:color w:val="FF0000"/>
                <w:sz w:val="28"/>
                <w:szCs w:val="28"/>
            </w:rPr>
            <w:t>NORMAL TEXT</w:t>
        </w:r>

    With ruby text:

        <w:r w:rsidRPr="003D52EF">
            <w:rPr>...</w:rPr>
            <w:ruby>
                <w:rubyPr>...</w:rubyPr>
                <w:rt>
                    <w:r w:rsidR="00FB4E90" w:rsidRPr="00FB4E90">
                        <w:rPr>
                            <w:rFonts w:ascii="Taigi Unicode" w:hAnsi="Taigi Unicode"/>     # nopep8
                            <w:color w:val="0000FF"/>
                            <w:sz w:val="20"/>
                            <w:szCs w:val="20"/>
                        </w:rPr>
                        <w:t>RUBY TEXT</w:t>
                    </w:r>
                </w:rt>
                <w:rubyBase>
                    <w:r w:rsidR="00D56347" w:rsidRPr="003D52EF">
                        <w:rPr>
                            <w:rFonts w:ascii="Taigi Unicode" w:hAnsi="Taigi Unicode"/>     # nopep8
                            <w:sz w:val="20"/>
                            <w:szCs w:val="20"/>
                        </w:rPr>
                        <w:t>RUBY BASE</w:t>
                    </w:r>
                </w:rubyBase>
            </w:ruby>
        </w:r>
    """
    x = {'o': 0}
    color_tag = '<w:color w:val="%s"/>' % color.upper()
    src = open(input).read()

    def satisfy(o):
        return o < 0x3000 and not (o >= 0x2000 and o <= 0x200f) and o != 0x20

    def repl(m):
        font, text = m.group(1), m.group(2)
        u = unicode(text, 'utf-8')
        o = ord(u[0])

        cond = satisfy(o)
        if not cond and len(u) > 1 and (o >= 0x2000 and o <= 0x200f):
            o = ord(u[1])
            cond = satisfy(o)

        if cond:
            # Clean up the color tags inside
            font = re.sub(r'<w:color w:val="\w{6}"/>', '', font)
            font = font + color_tag
            x['o'] = x['o'] + 1

        d = "<w:rPr>%s</w:rPr><w:t>%s</w:t>" % (font, text)
        return d

    dst = re.sub(r"<w:rPr>(.+?)</w:rPr><w:t>(.+?)</w:t>", repl, src)
    open(output, 'w').write(dst)
    return x['o']


if __name__ == '__main__':
    if len(sys.argv) != 3:
        print 'usage: addcolor xmlfile color'
        sys.exit(1)

    src = sys.argv[1]
    head, tail = os.path.split(src)
    base, ext = os.path.splitext(tail)
    dst = os.path.join(head, '%s-colored%s' % (base, ext))

    occurences = color_non_kanji_text(src, dst, sys.argv[2])
    print "%i occurences colored, output file: %s" % (occurences, dst)
